/**
 * External dependencies
 */
import styled from "@emotion/styled";

/**
 * WordPress dependencies
 */
import { __ } from "@wordpress/i18n";
import {
  Button,
  CheckboxControl,
  Spinner,
  Notice,
} from "@wordpress/components";
import { useState } from "@wordpress/element";

/**
 * Internal dependencies
 */
import { Widget, Fieldset } from "../../components";
import { log } from "../../utils";

const WidgetStyled = styled(Widget)`
  .fieldset__list {
    min-height: 60px;
    max-height: 300px;
    overflow: auto;
  }
`;

// Allowed blocks widget.
export const FeatureAllowedBlocksWidget = ({
  isLoading,
  featureName,
  blockTypes = [],
  title,
  description,
  selectedBlocks,
  setSelectedBlocks,
  updateSettings,
}) => {
  // Is current block selected.
  const isChecked = (checkingName) =>
    !!selectedBlocks.find(({ name }) => name === checkingName);

  const [messageData, setMessageData] = useState({
    type: "success",
    message: "",
  });

  // Save button.
  const saveActions = () => {
    const [isSaving, setIsSaving] = useState(false);
    return (
      <>
        <Button
          variant="primary"
          disabled={isSaving}
          onClick={(e) => {
            e.preventDefault();
            setIsSaving(true);
            updateSettings()
              .then(() => {
                setMessageData({
                  type: "success",
                  message: __("Setttings saved!", "block-enhancements"),
                });
              })
              .catch((error) => {
                log(error, "error");
                setMessageData({
                  type: "error",
                  message: __(
                    "Something went wrong, please contact the author for support!",
                    "block-enhancements"
                  ),
                });
              })
              .finally(() => {
                setIsSaving(false);
              });
          }}
        >
          {__("Update settings", "block-enhancements")}
        </Button>
        {isSaving && <Spinner />}
      </>
    );
  };

  return (
    <WidgetStyled title={title} renderFooter={saveActions} isFullRow={true}>
      <p>{description}</p>
      <Fieldset className="fieldset">
        <div className="fieldset__label">
          <strong>
            {__(
              "Choose which blocks should be supported.",
              "block-enhancements"
            )}
          </strong>
        </div>
        {isLoading ? (
          <Spinner />
        ) : (
          blockTypes &&
          blockTypes.length > 0 && (
            <fieldset>
              <CheckboxControl
                label={__("Toggle All", "block-enhancements")}
                checked={selectedBlocks.length === blockTypes.length}
                onChange={(checked) => {
                  if (checked) {
                    setSelectedBlocks(
                      [...blockTypes.map(({ name }) => ({ name }))],
                      featureName
                    );
                  } else {
                    setSelectedBlocks([], featureName);
                  }
                }}
              />
              <ul className="fieldset__list">
                {blockTypes.map((item) => (
                  <li key={item?.name}>
                    <CheckboxControl
                      label={`${item?.title} (${item?.name})`}
                      checked={isChecked(item?.name)}
                      onChange={(checked) => {
                        let newSelectedBlocks = [];
                        if (checked) {
                          const seletecItem = blockTypes.find(
                            ({ name }) => name === item?.name
                          );
                          newSelectedBlocks = [
                            ...selectedBlocks,
                            { name: seletecItem.name },
                          ];
                        } else {
                          newSelectedBlocks = selectedBlocks.filter(
                            ({ name }) => name !== item?.name
                          );
                        }

                        setSelectedBlocks([...newSelectedBlocks], featureName);
                      }}
                    />
                  </li>
                ))}
              </ul>
            </fieldset>
          )
        )}
      </Fieldset>
      {messageData && messageData?.message && (
        <Notice status={messageData?.type} isDismissible={false}>
          {messageData.message}
        </Notice>
      )}
    </WidgetStyled>
  );
};
